==> 11.5.3.tar.gz: lib/XfXWidgets/XfxUpdater.dart <==
part of xwidgetbase;

class XfxUpdater extends StatefulWidget {
  XfxUpdater({
    Key? key,
    required this.xMainAPP,
    required this.nextPage_WithOutUpdate,
    required this.nameAPK_APP,
    this.title_Text = "",
    this.title_Color,
    this.title_Style,
    this.onCancel,
  }) : super(key: key);
  final String? title_Text;
  final Color? title_Color;
  final TextStyle? title_Style;
  final XApp? xMainAPP;
  final String nameAPK_APP;
  final String? nextPage_WithOutUpdate;
  final Function()? onCancel;

  @override
  _XfxUpdaterState createState() => _XfxUpdaterState();
}

class _XfxUpdaterState extends State<XfxUpdater> {
  bool XUpdateEnabled = true;
  String latestVers = "";
  late XAzureStorage xb;

  Widget? page;
  @override
  void initState() {
    super.initState();
    xb = XAzureStorage.parse(widget.xMainAPP!.blob_ConnString!);
    XCheckForUpdates();
  }

  Future<Widget?> XCheckForUpdates() async {
    if (XUpdateEnabled.not()) return null; //david 05/10/2022 09:58:53

    if (widget.xMainAPP!.blob_ConnString! != null || widget.xMainAPP!.blob_ConnString! != "") {
      var appUpdatesHistoryFile = await xb.getBlob("xupdater/${widget.xMainAPP!.blob_nameAPP}.txt");
      if (appUpdatesHistoryFile.statusCode == 200) {
        var x = await appUpdatesHistoryFile.stream.bytesToString();
        if (x != "") {
          latestVers = x.GetLine(x, 0).split('\t')[0];
        }
        //se l'ultima versione è diversa da quella dell'app allora scarico l'apk dal blob
        if (latestVers != widget.xMainAPP!.app_Version) {
          setState(() {
            page = _buildUI();
          });
          // update_APP();
        } else {
          Navigator.pushNamed(context, widget.nextPage_WithOutUpdate!);
        }
        return page;
      }
    } else {
      Navigator.pushNamed(context, widget.nextPage_WithOutUpdate!);
      return null;
    }
  }

  update_APP() async {
    var downDirs = Directory('/storage/emulated/0/Download/${widget.xMainAPP!.app_Name.replaceAll(" ", "")}'); //await getExternalStorageDirectories(type: StorageDirectory.downloads);
    if (downDirs.existsSync().not()) {
      downDirs.create();
    } else {}
    var baseAppDir = downDirs;
    var outPath = baseAppDir.path; // "";
    var msg = "Ho trovato una nuova versione: ${latestVers}, la sto scaricando ... attendi prego.";
    print("${widget.xMainAPP!.blob_nameAPP} ${latestVers} DownloadToFile ... in ${outPath}");

    //Scarico l'apk da blob
    var x = await xb.getBlob("${widget.xMainAPP!.blob_nameAPP}/${widget.xMainAPP!.blob_nameAPP}-${latestVers}.apk");
    print(x.statusCode);

    var d = DateTime.now();
    var UIfile = await XLibStorage.CreateFile(widget.nameAPK_APP, extensionFile: "apk", path: outPath);

    print(UIfile!.path); //baseDir.path + "/" + "MELapp.apk"); //${sex.app_Version}
    var buffer = await x.stream.toBytes();
    await UIfile.writeAsBytes(buffer);
    var e = DateTime.now();
    print(e.difference(d).inSeconds.toString());

    // var file = await File(outPath + "/MELapp.apk");
    var u = Uri(scheme: "content", host: "it.spinwave.${widget.xMainAPP!.blob_nameAPP}.fileprovider", pathSegments: ["external_files", "${widget.nameAPK_APP}.apk"]);

    // Create an AndroidIntent to open the APK file
    final intent = await AndroidIntent(
      action: 'action_view',
      flags: [Flag.FLAG_GRANT_READ_URI_PERMISSION, Flag.FLAG_ACTIVITY_NEW_TASK],
      data: u.toString(),
      type: 'application/vnd.android.package-archive',
    );
    print(intent.data);
    await intent.launch();
    exit(0);
  }

  _buildUI() {
    bool isBusy = false;
    return Scaffold(
      backgroundColor: XColors.backgroundDark,
      body: StatefulBuilder(builder: (context, setState) {
        return Column(
          children: [
            Expanded(
                flex: 6,
                child: Container(
                    alignment: Alignment.topCenter,
                    margin: EdgeInsets.symmetric(horizontal: 15),
                    child: SingleChildScrollView(
                        child: Column(mainAxisSize: MainAxisSize.min, children: [
                      SizedBox(height: 60),
                      widget.title_Text != null
                          ? Row(children: [
                              Expanded(
                                  child: XContainerWithLabel(
                                widget.title_Text,
                                color: XColors.backgroundDark,
                                textStyle: widget.title_Style ?? XStyles.xStyTextForLargeTitle(widget.title_Color ?? XColors.foregroundLight),
                              ))
                            ])
                          : Container(),
                      SizedBox(height: 3),
                      Row(children: [
                        Expanded(
                            child: Row(children: [
                          Text("Ver. attuale", style: XStyles.xStyTextForDescr()),
                          Text(" ${widget.xMainAPP!.app_Version}", style: XStyles.xStyTextForSubLabel(Colors.yellow)),
                        ])),
                        SizedBox(width: 10),
                        Expanded(
                            child: Row(children: [
                          Text("Ver. in CLOUD", style: XStyles.xStyTextForDescr()),
                          Text(" ${latestVers}", style: XStyles.xStyTextForSubLabel(Colors.green)),
                        ])),
                      ]),
                      SizedBox(height: 20),
                      Container(child: XStatefulWidgetBase.LoaderIndicator(isBusy) ?? Container()),
                      SizedBox(height: 5),
                    ])))),
            Expanded(
              child: isBusy.not()
                  ? Container(
                      child: Row(children: [
                      Expanded(
                          child: XBtnbase(
                        height: 60,
                        label: "Continua \nsenza aggiornare",
                        onPressed: widget.onCancel,
                        label_Color: Colors.yellow,
                      )),
                      Expanded(
                          child: XBtnbase(
                        height: 60,
                        label: "Aggiorna",
                        onPressed: () async {
                          setState(() {
                            isBusy = true;
                          });
                          await update_APP();
                        },
                        label_Color: Colors.greenAccent,
                      )),
                    ]))
                  : Container(),
            )
          ],
        );
      }),
    );
  }

  @override
  Widget build(BuildContext context) {
    return page ?? Container();
  }
}

==> 11.5.4.tar.gz: lib/XfXWidgets/XfxUpdater.dart <==
part of xwidgetbase;

class XfxUpdater extends StatefulWidget {
  XfxUpdater({
    Key? key,
    required this.xMainAPP,
    required this.nextPage_WithOutUpdate,
    required this.nameAPK_APP,
    this.title_Text = "",
    this.title_Color,
    this.title_Style,
    this.onCancel,
  }) : super(key: key);
  final String? title_Text;
  final Color? title_Color;
  final TextStyle? title_Style;
  final XApp? xMainAPP;
  final String nameAPK_APP;
  final String? nextPage_WithOutUpdate;
  final Function()? onCancel;

  @override
  _XfxUpdaterState createState() => _XfxUpdaterState();
}

class _XfxUpdaterState extends State<XfxUpdater> {
  bool XUpdateEnabled = true;
  String latestVers = "";
  late XAzureStorage xb;

  Widget? page;
  @override
  void initState() {
    super.initState();
    xb = XAzureStorage.parse(widget.xMainAPP!.blob_ConnString!);
    XCheckForUpdates();
  }

  Future<Widget?> XCheckForUpdates() async {
    if (XUpdateEnabled.not()) return null; //david 05/10/2022 09:58:53

    if (widget.xMainAPP!.blob_ConnString! != null || widget.xMainAPP!.blob_ConnString! != "") {
      var appUpdatesHistoryFile = await xb.getBlob("xupdater/${widget.xMainAPP!.blob_nameAPP}.txt");
      if (appUpdatesHistoryFile.statusCode == 200) {
        var x = await appUpdatesHistoryFile.stream.bytesToString();
        if (x != "") {
          latestVers = x.GetLine(x, 0).split('\t')[0];
        }
        //se l'ultima versione è diversa da quella dell'app allora scarico l'apk dal blob
        if (latestVers != widget.xMainAPP!.app_Version) {
          setState(() {
            page = _buildUI();
          });
          // update_APP();
        } else {
          Navigator.pushNamed(context, widget.nextPage_WithOutUpdate!);
        }
        return page;
      }
    } else {
      Navigator.pushNamed(context, widget.nextPage_WithOutUpdate!);
      return null;
    }
  }

  update_APP() async {
    var downDirs = Directory('/storage/emulated/0/Download/${widget.xMainAPP!.app_Name.replaceAll(" ", "")}'); //await getExternalStorageDirectories(type: StorageDirectory.downloads);
    if (downDirs.existsSync().not()) {
      downDirs.create();
    } else {}
    var baseAppDir = downDirs;
    var outPath = baseAppDir.path; // "";
    var msg = "Ho trovato una nuova versione: ${latestVers}, la sto scaricando ... attendi prego.";
    print("${widget.xMainAPP!.blob_nameAPP} ${latestVers} DownloadToFile ... in ${outPath}");

    //Scarico l'apk da blob
    var x = await xb.getBlob("${widget.xMainAPP!.blob_nameAPP}/${widget.xMainAPP!.blob_nameAPP}-${latestVers}.apk");
    print(x.statusCode);

    var d = DateTime.now();
    var UIfile = await XLibStorage.CreateFile(widget.nameAPK_APP, extensionFile: "apk", path: outPath);

    print(UIfile!.path); //baseDir.path + "/" + "MELapp.apk"); //${sex.app_Version}
    var buffer = await x.stream.toBytes();
    await UIfile.writeAsBytes(buffer);
    var e = DateTime.now();
    print(e.difference(d).inSeconds.toString());

    // var file = await File(outPath + "/MELapp.apk");
    var u = Uri(scheme: "content", host: "/storage/emulated/0/Download/${widget.xMainAPP!.app_Name.replaceAll(" ", "")}/${widget.nameAPK_APP}.apk"); //it.spinwave.${widget.xMainAPP!.blob_nameAPP}.fileprovider  pathSegments: ["external_files", "${widget.nameAPK_APP}.apk"]

    // Create an AndroidIntent to open the APK file
    final intent = await AndroidIntent(
      action: 'action_view',
      flags: [Flag.FLAG_GRANT_READ_URI_PERMISSION, Flag.FLAG_ACTIVITY_NEW_TASK],
      data: u.toString(),
      type: 'application/vnd.android.package-archive',
    );
    print(intent.data);
    await intent.launch();
    exit(0);
  }

  _buildUI() {
    bool isBusy = false;
    return Scaffold(
      backgroundColor: XColors.backgroundDark,
      body: StatefulBuilder(builder: (context, setState) {
        return Column(
          children: [
            Expanded(
                flex: 6,
                child: Container(
                    alignment: Alignment.topCenter,
                    margin: EdgeInsets.symmetric(horizontal: 15),
                    child: SingleChildScrollView(
                        child: Column(mainAxisSize: MainAxisSize.min, children: [
                      SizedBox(height: 60),
                      widget.title_Text != null
                          ? Row(children: [
                              Expanded(
                                  child: XContainerWithLabel(
                                widget.title_Text,
                                color: XColors.backgroundDark,
                                textStyle: widget.title_Style ?? XStyles.xStyTextForLargeTitle(widget.title_Color ?? XColors.foregroundLight),
                              ))
                            ])
                          : Container(),
                      SizedBox(height: 3),
                      Row(children: [
                        Expanded(
                            child: Row(children: [
                          Text("Ver. attuale", style: XStyles.xStyTextForDescr()),
                          Text(" ${widget.xMainAPP!.app_Version}", style: XStyles.xStyTextForSubLabel(Colors.yellow)),
                        ])),
                        SizedBox(width: 10),
                        Expanded(
                            child: Row(children: [
                          Text("Ver. in CLOUD", style: XStyles.xStyTextForDescr()),
                          Text(" ${latestVers}", style: XStyles.xStyTextForSubLabel(Colors.green)),
                        ])),
                      ]),
                      SizedBox(height: 20),
                      Container(child: XStatefulWidgetBase.LoaderIndicator(isBusy) ?? Container()),
                      SizedBox(height: 5),
                    ])))),
            Expanded(
              child: isBusy.not()
                  ? Container(
                      child: Row(children: [
                      Expanded(
                          child: XBtnbase(
                        height: 60,
                        label: "Continua \nsenza aggiornare",
                        onPressed: widget.onCancel,
                        label_Color: Colors.yellow,
                      )),
                      Expanded(
                          child: XBtnbase(
                        height: 60,
                        label: "Aggiorna",
                        onPressed: () async {
                          setState(() {
                            isBusy = true;
                          });
                          await update_APP();
                        },
                        label_Color: Colors.greenAccent,
                      )),
                    ]))
                  : Container(),
            )
          ],
        );
      }),
    );
  }

  @override
  Widget build(BuildContext context) {
    return page ?? Container();
  }
}
